/***** Proiect PM : LineFollower ******
	Studenti:
		Florin Tifrea
		Alexandru Duse-Vasiliu
	Grupa:  335CC
***************************************/
#include <avr\io.h>
#include <avr\interrupt.h>
#define F_CPU 16000000UL
#include <util/delay.h>
#define SPEED0 0		
#define SPEED1 80
#define SPEED2 102
#define SPEED3 127

unsigned char left,right;
int directie;

/* Initializare ADC*/
void ADC_init(void)
{
    /* referinta AVCC cu condesator extern pe pinul AREF */
    ADMUX = (1 << REFS0);

    /* activeaza ADC si il pune in modul free running (auto trigger)
       prescaler de ceas la maximum
       FARA intrerupere la conversie completa */	
    ADCSRA = (1 << ADEN) | (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0);
}



/*  initializare PWM  */
void pwm_init(){
	/* sensors */
	/* input */
	PORTA = 0xff;
	DDRA = 0x00;
	/* PORTB = h-bridge enable*/
	/* output PB3 */
	PORTB = _BV(PB3);
	DDRB = 0xff;
	
	/* PORTC = h-bridge inputs */
	/* output */
	PORTC = 0x00;
	DDRC = 0xff;
	
	/* PORTD = h-bridge enable */
	/* output PD7 */
	PORTD = _BV(PD7);
	DDRD = 0xff;
	
	/* init timer */
	
    /* timer 0 */
    TCCR0 = _BV(WGM00) | _BV(WGM01) | _BV(COM01) | _BV(CS00);
    OCR0 = 0;
	
	/* timer 2 */
    TCCR2 = _BV(WGM20) | _BV(WGM21) | _BV(COM21) | _BV(CS20);
    OCR2 = 0;
	
}

/*    setare viteza pe PWM       */
void pwm_set_speed( int left_speed, int right_speed )
{
     OCR0 = left_speed;
     OCR2 = right_speed;
} 


/*    calculare viteza in functie de directie    */
void setare_viteza( int direction)
{
	/* direction=left-right */
	/* left si right sunt valorile primite de la senzori */
	/*  si reprezinta numarul de senzori ce au vazut negru */
	
    switch(direction){
	case -3:
		/* reverse right */
		PORTC=(1<<PC3)|(1<<PC2)|(1<<PC0);
		pwm_set_speed(SPEED3,SPEED1);
		break;
	case -2:
		/* break right */
		PORTC=(1<<PC3)|(1<<PC2)|(1<<PC1)|(1<<PC0);
		pwm_set_speed(SPEED3,255);
		break;
	case -1:
		/* free run right */
		PORTC=(1<<PC3)|(1<<PC2)|(1<<PC1);
		pwm_set_speed(SPEED3,SPEED0);
		break;
	case 0:
		/* foreward */
		PORTC=(1<<PC3)|(1<<PC2)|(1<<PC1);
		pwm_set_speed(SPEED3,SPEED3);
		break;
	case 1:
		/* free run left */
		PORTC=(1<<PC3)|(1<<PC2)|(1<<PC1);
		pwm_set_speed(SPEED0,SPEED3);
		break;
	case 2:
		/* break left */
		PORTC=(1<<PC3)|(1<<PC2)|(1<<PC1)|(1<<PC0);
		pwm_set_speed(255,SPEED3);
		break;
	case 3:
		/* reverse left */
		PORTC=(1<<PC3)|(1<<PC2)|(1<<PC1);
		pwm_set_speed(SPEED1,SPEED3);
		break;
	
	case 5:
		/* rotatie */
		PORTC=(1<<PC3)|(1<<PC2)|(1<<PC0);
		pwm_set_speed(SPEED2,SPEED2);
		break;
	default:
		PORTC=(1<<PC3)|(1<<PC2)|(1<<PC1);
		pwm_set_speed(SPEED0,SPEED0);
	}
		
} 




/* functie pentru obtinerea datelor de la banda de senzori(PORTA) */
void getSenzorData(){
	left=0;
	right=0;
	/* daca toti 3 senzorii sunt pe negru */
	if(PINA &(1) && PINA &(1<<1) && PINA &(1<<2)){
		left=3;
	}else
		/*daca doar 2 senzori sunt pe negru*/
		if(PINA &(1<<1) && PINA &(1<<2)){
			left=2;
		}else 
			/*   daca primul senzor este pe negru    */
			if(PINA &(1<<2)){
				left=1;
			}
	
	/*daca ultimi trei senzori sunt pe negru*/
	if(PINA &(1<<7) && PINA &(1<<6) && PINA &(1<<5)){
		right=3;
	}else 
		/*daca ultimi 2 senzori sunt pe negru*/
		if(PINA &(1<<6) && PINA &(1<<5)){
			right=2;
		}else 
			/* daca ultimul senzor este pe negru */
			if(PINA &(1<<5)){
				right=1;
			}
			
}


int main()
{

	int i;
	int activ=0; /* pentru a verifica daca robotul a terminat sau nu traseul */
	int direction = 0, old_direction;
	
	/* initiere ADC */
	ADC_init();
	/* initiere PWM */
	pwm_init();
	/* initiere viteza */
	setare_viteza(10);
	
	while(1){
		
		/* obtinere date de la senzori */
		if(activ==1){
			getSenzorData();
			PORTC=PINA;
			if(PINA==0){
				if(direction!=5){
					direction=5;
					setare_viteza(direction);
				}
			}else{
			
				if(direction==5){
					PORTD=0x4A;
				}
				old_direction=direction;
				direction=left-right;
				if(old_direction!=direction)
					setare_viteza(direction);
			}
		}
			
		
		if(!(PINA &(1<<6))){
			activ^=1;
			if(activ==0){
				/* stop */
				direction=10;
				setare_viteza(10);
			}
			for(i=0;i<20;i++){
				_delay_ms(120);
			
			}
		}
	}

	return 0;
}
